#ifndef __CyberDevice_DynamicPortReceiver_h__
#define __CyberDevice_DynamicPortReceiver_h__

#include "DynamicPortReceiver.h"
#include <conio.h>

using CyberDevice::DynamicPortReceiver;

//--------------------------------------------------------------------------------------------------InterfaceImpl controller functions
void DynamicPortReceiver::addCustomProvider(const char* name)
{
	std::cout << "[CALLED] DynamicPortReceiver::addCustomProvider\n";

	// creating custom port&service
	mpCustomServiceImpl = new CustomDynamicServiceImpl(this);
	mCustomPort.registerProvider(name , "Tester::CustomDynamicService", *mpCustomServiceImpl);
	addPort(mCustomPort);

	// without this call, connection to newly created service remains inactive, as DynamicPortTester RTC can tell
	//m_portAdmin.activatePorts(); // this is valid only if RTC::RTObject is modified. (m_portAdmin made public)
}
RTC::PortService_ptr DynamicPortReceiver::getPortInfo(const char* name)
{
	// future plan:
	// finding appropriate port
	//RTC::PortServiceList ans = *(this->get_ports());
	//for (unsigned i = 0; i < ans.length(); i++)
	//{
	//	if (ans[i]->get_port_profile()->name == name)
	//	{
	//		return ans[i];
	//	}
	//}

	// yet only return specific port
	return mCustomPort.getPortRef();
}
void DynamicPortReceiver::notify_function_called()
{
	std::cout << "\t\t *** Function SUCCESSFULLY called! *** \n";
}
//--------------------------------------------------------------------------------------------------RTC
DynamicPortReceiver::DynamicPortReceiver(RTC::Manager *_manager)
	: RTC::DataFlowComponentBase(_manager),
	mPortServicePort("TestServicePort"),
	mCustomPort("CustomServicePort")
{
	mpPortServiceImpl = new PortServiceInfoImpl(this);
	mPortServicePort.registerProvider("PortServiceInfoTest", "Tester::PortServiceInfo", *mpPortServiceImpl);
	addPort(mPortServicePort);
}
DynamicPortReceiver::~DynamicPortReceiver()
{

}
const char* dyn_port_rec_spec[] =
{
	"implementation_id", "DynamicPortReceiver",
	"type_name",         "DynamicPortReceiver",
	"description",       "",
	"version",           "1.0.0",
	"vendor",            "Gyorgy Persa",
	"category",          "VirCA CyberDevice",
	"activity_type",     "EVENT_DRIVEN",
	"kind",              "DataFlowComponent",
	"max_instance",      "1",
	"language",          "C++",
	"lang_type",         "compile",
	"exec_cxt.periodic.rate", "1.0",
	""
};
void DynamicPortReceiver::AdapterInit(RTC::Manager* manager)
{
	coil::Properties profile(dyn_port_rec_spec);
	manager->registerFactory(profile,
		RTC::Create<DynamicPortReceiver>,
		RTC::Delete<DynamicPortReceiver>);

	manager->createComponent("DynamicPortReceiver");

	// multiple provider - 1 consumer ?
// 	manager->createComponent("DynamicPortReceiver");
// 	manager->createComponent("DynamicPortReceiver");
}
//--------------------------------------------------------------------------------------------------
RTC::ReturnCode_t DynamicPortReceiver::onStartup(RTC::UniqueId exec_handle)
{
	return RTC::RTC_OK;
}
RTC::ReturnCode_t DynamicPortReceiver::onExecute(RTC::UniqueId exec_handle)
{
	return RTC::RTC_OK;
}
RTC::ReturnCode_t DynamicPortReceiver::onActivated(RTC::UniqueId exec_handle)
{
	return RTC::RTC_OK;
}
RTC::ReturnCode_t DynamicPortReceiver::onDeactivated(RTC::UniqueId exec_handle)
{
	return RTC::RTC_OK;
}
RTC::ReturnCode_t DynamicPortReceiver::onReset(RTC::UniqueId exec_handle)
{
	return RTC::RTC_OK;
}
//--------------------------------------------------------------------------------------------------main
int main(int argc, char** argv)
{
	// RTC
	//Creating RT component
	RTC::Manager* manager;
	manager = RTC::Manager::init(argc, argv);
	manager->init(argc, argv);
	manager->setModuleInitProc(CyberDevice::DynamicPortReceiver::AdapterInit);
	manager->activateManager();
	manager->runManager(true);

	std::cout << "Press 'ESC' to exit!\n";
	char ch = '\0';
	while (ch != 27) // ESCAPE
		ch = _getch();

	manager->shutdown();
	return 0;
}

#endif // !__CyberDevice_DynamicPortReceiver_h__